﻿using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace PropertyGrid.Source
{
    public class GenericPropertyChanged : INotifyPropertyChanged
    {
        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Called when [property changed].
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var handler = this.PropertyChanged;
            handler?.Invoke(this, new PropertyChangedEventArgs(propertyName));

            NotifyPropertyChanged(propertyName);
        }

        /// <summary>
        /// Notifies the property changed.
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected virtual void NotifyPropertyChanged(string propertyName)
        {
        }

        /// <summary>
        /// Updates the specified property name.
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        public void Update(string propertyName)
        {
            OnPropertyChanged(propertyName);
        }
    }
}
